@@ -14,15 +14,19 @@ import os
 import os.path
 import re
 import stat
+from abc import ABC, abstractmethod
 from contextlib import suppress
+from logging import Logger
 from pathlib import Path
 from textwrap import dedent
 from typing import Tuple
 
 from cloudinit import log as logging
 from cloudinit import subp, temp_utils, util
+from cloudinit.cloud import Cloud
+from cloudinit.config import Config
 from cloudinit.config.schema import MetaSchema, get_meta_doc
-from cloudinit.distros import ALL_DISTROS
+from cloudinit.distros import ALL_DISTROS, Distro
 from cloudinit.settings import PER_ALWAYS
 
 MODULE_DESCRIPTION = """\
@@ -81,6 +85,7 @@ meta: MetaSchema = {
             """
         ),
     ],
+    "activate_by_schema_keys": [],
 }
 
 __doc__ = get_meta_doc(meta)
@@ -94,7 +99,7 @@ DEFAULT_CONFIG = {
 KEYDATA_PATH = Path("/cc_growpart_keydata")
 
 
-class RESIZE(object):
+class RESIZE:
     SKIPPED = "SKIPPED"
     CHANGED = "CHANGED"
     NOCHANGE = "NOCHANGE"
@@ -104,11 +109,11 @@ class RESIZE(object):
 LOG = logging.getLogger(__name__)
 
 
-def resizer_factory(mode):
+def resizer_factory(mode: str, distro: Distro):
     resize_class = None
     if mode == "auto":
         for (_name, resizer) in RESIZERS:
-            cur = resizer()
+            cur = resizer(distro)
             if cur.available():
                 resize_class = cur
                 break
@@ -124,7 +129,7 @@ def resizer_factory(mode):
         if mode not in mmap:
             raise TypeError("unknown resize mode %s" % mode)
 
-        mclass = mmap[mode]()
+        mclass = mmap[mode](distro)
         if mclass.available():
             resize_class = mclass
 
@@ -138,7 +143,20 @@ class ResizeFailedException(Exception):
     pass
 
 
-class ResizeGrowPart(object):
+class Resizer(ABC):
+    def __init__(self, distro: Distro):
+        self._distro = distro
+
+    @abstractmethod
+    def available(self) -> bool:
+        ...
+
+    @abstractmethod
+    def resize(self, diskdev, partnum, partdev):
+        ...
+
+
+class ResizeGrowPart(Resizer):
     def available(self):
         myenv = os.environ.copy()
         myenv["LANG"] = "C"
@@ -159,7 +177,8 @@ class ResizeGrowPart(object):
 
         # growpart uses tmp dir to store intermediate states
         # and may conflict with systemd-tmpfiles-clean
-        with temp_utils.tempdir(needs_exe=True) as tmpd:
+        tmp_dir = self._distro.get_tmp_exec_path()
+        with temp_utils.tempdir(dir=tmp_dir, needs_exe=True) as tmpd:
             growpart_tmp = os.path.join(tmpd, "growpart")
             if not os.path.exists(growpart_tmp):
                 os.mkdir(growpart_tmp, 0o700)
@@ -188,7 +207,7 @@ class ResizeGrowPart(object):
         return (before, get_size(partdev))
 
 
-class ResizeGpart(object):
+class ResizeGpart(Resizer):
     def available(self):
         myenv = os.environ.copy()
         myenv["LANG"] = "C"
@@ -346,7 +365,7 @@ def is_encrypted(blockdev, partition) -> bool:
 
 def get_underlying_partition(blockdev):
     command = ["dmsetup", "deps", "--options=devname", blockdev]
-    dep: str = subp.subp(command)[0]  # type: ignore
+    dep: str = subp.subp(command)[0]  # pyright: ignore
     # Returned result should look something like:
     # 1 dependencies : (vdb1)
     if not dep.startswith("1 depend"):
@@ -547,7 +566,9 @@ def resize_devices(resizer, devices):
     return info
 
 
-def handle(_name, cfg, _cloud, log, _args):
+def handle(
+    name: str, cfg: Config, cloud: Cloud, log: Logger, args: list
+) -> None:
     if "growpart" not in cfg:
         log.debug(
             "No 'growpart' entry in cfg.  Using default: %s" % DEFAULT_CONFIG
@@ -581,7 +602,7 @@ def handle(_name, cfg, _cloud, log, _args):
         return
 
     try:
-        resizer = resizer_factory(mode)
+        resizer = resizer_factory(mode, cloud.distro)
     except (ValueError, TypeError) as e:
         log.debug("growpart unable to find resizer for '%s': %s" % (mode, e))
         if mode != "auto":
