@@ -8,10 +8,13 @@
 
 """Resolv Conf: configure resolv.conf"""
 
+from logging import Logger
 from textwrap import dedent
 
 from cloudinit import log as logging
 from cloudinit import templater, util
+from cloudinit.cloud import Cloud
+from cloudinit.config import Config
 from cloudinit.config.schema import MetaSchema, get_meta_doc
 from cloudinit.settings import PER_INSTANCE
 
@@ -52,7 +55,15 @@ meta: MetaSchema = {
     "name": "Resolv Conf",
     "title": "Configure resolv.conf",
     "description": MODULE_DESCRIPTION,
-    "distros": ["alpine", "fedora", "opensuse", "photon", "rhel", "sles"],
+    "distros": [
+        "alpine",
+        "fedora",
+        "mariner",
+        "opensuse",
+        "photon",
+        "rhel",
+        "sles",
+    ],
     "frequency": PER_INSTANCE,
     "examples": [
         dedent(
@@ -104,7 +115,9 @@ def generate_resolv_conf(template_fn, params, target_fname):
     templater.render_to_file(template_fn, target_fname, params)
 
 
-def handle(name, cfg, cloud, log, _args):
+def handle(
+    name: str, cfg: Config, cloud: Cloud, log: Logger, args: list
+) -> None:
     """
     Handler for resolv.conf
 
