@@ -1,28 +1,24 @@
 # This file is part of cloud-init. See LICENSE file for license information.
 
+import logging
+import os
 import re
 from io import StringIO
 
 import pytest
 
-from cloudinit import util
-from cloudinit.config.cc_snap import (
-    ASSERTIONS_FILE,
-    add_assertions,
-    handle,
-    run_commands,
-)
+from cloudinit import helpers, util
+from cloudinit.config.cc_snap import add_assertions, handle, run_commands
 from cloudinit.config.schema import (
     SchemaValidationError,
     get_schema,
     validate_cloudconfig_schema,
 )
-from tests.unittests.helpers import (
-    CiTestCase,
-    mock,
-    skipUnlessJsonSchema,
-    wrap_and_call,
-)
+from tests.unittests.helpers import CiTestCase, mock, skipUnlessJsonSchema
+from tests.unittests.util import get_cloud
+
+M_PATH = "cloudinit.config.cc_snap."
+ASSERTIONS_FILE = "/var/lib/cloud/instance/snapd.assertions"
 
 SYSTEM_USER_ASSERTION = """\
 type: system-user
@@ -91,98 +87,81 @@ IQsRTONp+iVS8YxSmoYZjDlCgRMWUmawez/Fv5b9Fb/XkO5Eq4e+KfrpUujXItaipb+tV8h5v3t
 oG3Ie3WOHrVjCLXIdYslpL1O4nadqR6Xv58pHj6k"""
 
 
-class FakeCloud(object):
-    def __init__(self, distro):
-        self.distro = distro
+@pytest.fixture()
+def fake_cloud(tmpdir):
+    paths = helpers.Paths(
+        {
+            "cloud_dir": tmpdir.join("cloud"),
+            "run_dir": tmpdir.join("cloud-init"),
+            "templates_dir": tmpdir.join("templates"),
+        }
+    )
+    cloud = get_cloud(paths=paths)
+    yield cloud
 
 
-class TestAddAssertions(CiTestCase):
-
-    with_logs = True
-
-    def setUp(self):
-        super(TestAddAssertions, self).setUp()
-        self.tmp = self.tmp_dir()
-
+class TestAddAssertions:
     @mock.patch("cloudinit.config.cc_snap.subp.subp")
-    def test_add_assertions_on_empty_list(self, m_subp):
+    def test_add_assertions_on_empty_list(self, m_subp, caplog, tmpdir):
         """When provided with an empty list, add_assertions does nothing."""
-        add_assertions([])
-        self.assertEqual("", self.logs.getvalue())
-        m_subp.assert_not_called()
+        assert_file = tmpdir.join("snapd.assertions")
+        add_assertions([], assert_file)
+        assert not caplog.text
+        assert 0 == m_subp.call_count
 
-    def test_add_assertions_on_non_list_or_dict(self):
+    def test_add_assertions_on_non_list_or_dict(self, tmpdir):
         """When provided an invalid type, add_assertions raises an error."""
-        with self.assertRaises(TypeError) as context_manager:
-            add_assertions(assertions="I'm Not Valid")
-        self.assertEqual(
-            "assertion parameter was not a list or dict: I'm Not Valid",
-            str(context_manager.exception),
-        )
+        assert_file = tmpdir.join("snapd.assertions")
+        with pytest.raises(
+            TypeError,
+            match="assertion parameter was not a list or dict: I'm Not Valid",
+        ):
+            add_assertions("I'm Not Valid", assert_file)
 
     @mock.patch("cloudinit.config.cc_snap.subp.subp")
-    def test_add_assertions_adds_assertions_as_list(self, m_subp):
+    def test_add_assertions_adds_assertions_as_list(
+        self, m_subp, caplog, tmpdir
+    ):
         """When provided with a list, add_assertions adds all assertions."""
-        self.assertEqual(
-            ASSERTIONS_FILE, "/var/lib/cloud/instance/snapd.assertions"
-        )
-        assert_file = self.tmp_path("snapd.assertions", dir=self.tmp)
+        assert_file = tmpdir.join("snapd.assertions")
         assertions = [SYSTEM_USER_ASSERTION, ACCOUNT_ASSERTION]
-        wrap_and_call(
-            "cloudinit.config.cc_snap",
-            {"ASSERTIONS_FILE": {"new": assert_file}},
-            add_assertions,
-            assertions,
-        )
-        self.assertIn(
-            "Importing user-provided snap assertions", self.logs.getvalue()
-        )
-        self.assertIn("sertions", self.logs.getvalue())
-        self.assertEqual(
-            [mock.call(["snap", "ack", assert_file], capture=True)],
-            m_subp.call_args_list,
-        )
-        compare_file = self.tmp_path("comparison", dir=self.tmp)
+        add_assertions(assertions, assert_file)
+        assert "Importing user-provided snap assertions" in caplog.text
+        assert "sertions" in caplog.text
+        assert [
+            mock.call(["snap", "ack", assert_file], capture=True)
+        ] == m_subp.call_args_list
+        compare_file = tmpdir.join("comparison")
         util.write_file(compare_file, "\n".join(assertions).encode("utf-8"))
-        self.assertEqual(
-            util.load_file(compare_file), util.load_file(assert_file)
-        )
+        assert util.load_file(compare_file) == util.load_file(assert_file)
 
     @mock.patch("cloudinit.config.cc_snap.subp.subp")
-    def test_add_assertions_adds_assertions_as_dict(self, m_subp):
+    def test_add_assertions_adds_assertions_as_dict(
+        self, m_subp, caplog, tmpdir
+    ):
         """When provided with a dict, add_assertions adds all assertions."""
-        self.assertEqual(
-            ASSERTIONS_FILE, "/var/lib/cloud/instance/snapd.assertions"
-        )
-        assert_file = self.tmp_path("snapd.assertions", dir=self.tmp)
+        assert_file = tmpdir.join("snapd.assertions")
         assertions = {"00": SYSTEM_USER_ASSERTION, "01": ACCOUNT_ASSERTION}
-        wrap_and_call(
-            "cloudinit.config.cc_snap",
-            {"ASSERTIONS_FILE": {"new": assert_file}},
-            add_assertions,
-            assertions,
-        )
-        self.assertIn(
-            "Importing user-provided snap assertions", self.logs.getvalue()
-        )
-        self.assertIn(
-            "DEBUG: Snap acking: ['type: system-user', 'authority-id: Lqv",
-            self.logs.getvalue(),
-        )
-        self.assertIn(
-            "DEBUG: Snap acking: ['type: account-key', 'authority-id: canonic",
-            self.logs.getvalue(),
-        )
-        self.assertEqual(
-            [mock.call(["snap", "ack", assert_file], capture=True)],
-            m_subp.call_args_list,
-        )
-        compare_file = self.tmp_path("comparison", dir=self.tmp)
+        add_assertions(assertions, assert_file)
+        assert "Importing user-provided snap assertions" in caplog.text
+        assert (
+            M_PATH[:-1],
+            logging.DEBUG,
+            "Snap acking: ['type: system-user', 'authority-id: "
+            "LqvZQdfyfGlYvtep4W6Oj6pFXP9t1Ksp']",
+        ) in caplog.record_tuples
+        assert (
+            M_PATH[:-1],
+            logging.DEBUG,
+            "Snap acking: ['type: account-key', 'authority-id: canonical']",
+        ) in caplog.record_tuples
+        assert [
+            mock.call(["snap", "ack", assert_file], capture=True)
+        ] == m_subp.call_args_list
+        compare_file = tmpdir.join("comparison")
         combined = "\n".join(assertions.values())
         util.write_file(compare_file, combined.encode("utf-8"))
-        self.assertEqual(
-            util.load_file(compare_file), util.load_file(assert_file)
-        )
+        assert util.load_file(compare_file) == util.load_file(assert_file)
 
 
 class TestRunCommands(CiTestCase):
@@ -339,37 +318,21 @@ class TestSnapSchema:
                 validate_cloudconfig_schema(config, get_schema(), strict=True)
 
 
-class TestHandle(CiTestCase):
-
-    with_logs = True
-
-    def setUp(self):
-        super(TestHandle, self).setUp()
-        self.tmp = self.tmp_dir()
-
+class TestHandle:
     @mock.patch("cloudinit.config.cc_snap.subp.subp")
-    def test_handle_adds_assertions(self, m_subp):
+    def test_handle_adds_assertions(self, m_subp, fake_cloud, tmpdir):
         """Any configured snap assertions are provided to add_assertions."""
-        assert_file = self.tmp_path("snapd.assertions", dir=self.tmp)
-        compare_file = self.tmp_path("comparison", dir=self.tmp)
+        assert_file = os.path.join(
+            fake_cloud.paths.get_ipath_cur(), "snapd.assertions"
+        )
+        compare_file = tmpdir.join("comparison")
         cfg = {
             "snap": {"assertions": [SYSTEM_USER_ASSERTION, ACCOUNT_ASSERTION]}
         }
-        wrap_and_call(
-            "cloudinit.config.cc_snap",
-            {"ASSERTIONS_FILE": {"new": assert_file}},
-            handle,
-            "snap",
-            cfg=cfg,
-            cloud=None,
-            log=self.logger,
-            args=None,
-        )
+        handle("snap", cfg=cfg, cloud=fake_cloud, log=mock.Mock(), args=None)
         content = "\n".join(cfg["snap"]["assertions"])
         util.write_file(compare_file, content.encode("utf-8"))
-        self.assertEqual(
-            util.load_file(compare_file), util.load_file(assert_file)
-        )
+        assert util.load_file(compare_file) == util.load_file(assert_file)
 
 
 # vi: ts=4 expandtab
