@@ -12,7 +12,11 @@ def test_init(m_os, with_pkgin):
     m_os.path.exists.return_value = with_pkgin
     cfg = {}
 
-    distro = cloudinit.distros.netbsd.NetBSD("netbsd", cfg, None)
+    # patch ifconfig -a
+    with mock.patch(
+        "cloudinit.distros.networking.subp.subp", return_value=("", None)
+    ):
+        distro = cloudinit.distros.netbsd.NetBSD("netbsd", cfg, None)
     expectation = ["pkgin", "-y", "full-upgrade"] if with_pkgin else None
     assert distro.pkg_cmd_upgrade_prefix == expectation
     assert [mock.call("/usr/pkg/bin/pkgin")] == m_os.path.exists.call_args_list
